import { Graph } from "./graph.js";
import { Behavior, BehaviorBuilder } from "./behavior.js";
import { Moment, Resource, State } from "./resource.js";
import { RelinkingOrder } from "./common.js";
export declare enum ExtentRemoveStrategy {
    extentOnly = 0,
    containedLifetimes = 1
}
declare class ExtentLifetime {
    addedToGraphWhen: number | null;
    extents: Set<Extent>;
    children: Set<ExtentLifetime> | null;
    parent: ExtentLifetime | null;
    constructor(extent: Extent);
    unify(extent: Extent): void;
    addChild(extent: Extent): void;
    addChildLifetime(lifetime: ExtentLifetime): void;
    hasCompatibleLifetime(lifetime: ExtentLifetime | null): boolean;
    getAllContainedExtents(): Extent[];
    getAllContainingExtents(): Extent[];
}
export declare class Extent {
    debugConstructorName: string | undefined;
    debugName: string | undefined;
    behaviors: Behavior[];
    resources: Resource[];
    graph: Graph;
    addedToGraphWhen: number | null;
    addedToGraph: State<boolean>;
    lifetime: ExtentLifetime | null;
    unsubscribes: Set<() => void>;
    static readonly removeContainedLifetimes = ExtentRemoveStrategy.containedLifetimes;
    static readonly relinkingOrderSubsequent = RelinkingOrder.relinkingOrderSubsequent;
    constructor(graph: Graph);
    debugHere(): string;
    unifyLifetime<T extends Extent>(extent: T): void;
    addChildLifetime<T extends Extent>(extent: T): void;
    hasCompatibleLifetime<T extends Extent>(extent: T): boolean;
    addBehavior(behavior: Behavior): void;
    addResource(resource: Resource): void;
    addToGraphWithAction(debugName?: string): void;
    addToGraph(): void;
    removeFromGraphWithAction(strategy?: ExtentRemoveStrategy, debugName?: string): void;
    removeFromGraph(strategy?: ExtentRemoveStrategy): void;
    subscribeToJustUpdated(resources: Resource[], callback: (ext: this) => void): () => void;
    unsubscribeAll(): void;
    nameResources(): void;
    behavior(): BehaviorBuilder<this>;
    resource(name?: string): Resource;
    moment<T>(name?: string): Moment<T>;
    state<T>(initialState: T, name?: string): State<T>;
    sideEffect(block: (ext: this) => void, debugName?: string): void;
    actionAsync(action: (ext: this) => void, debugName?: string): Promise<unknown>;
    action(action: (ext: this) => void, debugName?: string): void;
}
export {};
